An example dataset as seen in table 1 from kornbrot1990;textualrankdifferencetest.
The time per problem was recorded for each subject under placebo and drug conditions for the purpose of measuring alertness.
Usage
kornbrot_table1
Arguments
Format
A data frame with 13 rows and 3 variables:
subject
Subject identifier
placebo
The time required to complete a task under the placebo condition
drug
The time required to complete a task under the drug condition